#!/bin/bash
set -ex

OURTMPDIR=$(mktemp -d "$TMPDIR"signappXXX)
OUTPUTDIR=${TMPDIR}
RETURNDIR=$(pwd)
PLIST=/usr/libexec/PlistBuddy
CODESIGN=/usr/bin/codesign

# We no longer want to set CODESIGN_ALLOCATE - with this set, this now
# fails unless additional arguments are passed.
#export CODESIGN_ALLOCATE=/usr/bin/codesign_allocate

function absolutePathForFile
{
    pushd "$(dirname "$1")" > /dev/null 2>&1
    echo $(pwd)/$(basename "$1")
    popd > /dev/null 2>&1
}

function cleanup
{
    cd "$RETURNDIR"
    rm -rf "$OURTMPDIR"
}

function cleanupAndExit
{
    cleanup
    exit $1
}

function fileExists
{
    if [ "$1" == "" ]
    then
        return
    fi

    if [ ! -f "$1" ]
    then
        echo "File \"$1\" could not be found."
        cleanupAndExit -1
    fi
}

SCRIPTDIR=$(absolutePathForFile "$0")
SCRIPTDIR=$(dirname "$SCRIPTDIR")

CONFIGFILES=()
BUNDLEFILES=()
PACKAGEAPP=${SCRIPTDIR}/PackageApplication.pl

while [ $# -gt 1 ]
do

case "$1" in 

--signid)   shift
            SIGNID=$1
            ;;

--prefix)   shift
            PREFIX=$1
            ;;

--bundleid) shift
            BUNDLEID=$1
            ;;

--bundlefile)   shift
            BUNDLEFILES+=( "$(absolutePathForFile "$1")" )
            ;;

--configfile)   shift
            CONFIGFILES+=( "$(absolutePathForFile "$1")" )
            ;;

--ttconfig) shift
            TTCONFIG=$(absolutePathForFile "$1")
            ;;

--bundledisplay)   shift
            BUNDLEDISPLAY=$1
            ;;

--bundlename)   shift
            BUNDLENAME=$1
            ;;

--provision)   shift
            PROVISION=$1
            ;;

--outputipa) shift
            IPANAME=$(absolutePathForFile "$1")
            ;;

--inputipa)  shift
            INPUTIPA=$(absolutePathForFile "$1")
            ;;

--acctcode) shift
            ACCTCODE="$1"
            ;;
--settings) shift
            SETTINGS="$1"
            ;;

--codesign) shift
#            CODESIGN="$1"
            ;;

--codesign_allocate) shift
# See above note on CODESIGN_ALLOCATE
#            export CODESIGN_ALLOCATE="$1"
            ;;

esac
shift
done

pushd "$OURTMPDIR" > /dev/null 2>&1

fileExists "$INPUTIPA"
fileExists "$PROVISION"
fileExists "$TTCONFIG"

for configFile in "${CONFIGFILES[@]}"
do
    fileExists "$configFile"
done

for bundleFile in "${BUNDLEFILES[@]}"
do
    fileExists "$bundleFile"
done


#Unpack input ipa file, move .app file up to our tmp dir and get rid of the Payload

unzip "$INPUTIPA" || cleanupAndExit $?
mv Payload/*.app .
rm -rf Payload

FILES=(*.app)
FILE=${FILES[0]}

TMPDIR=`mktemp -d apppackXXX`
APPNAME=$(basename "$FILE" .app)
ORIGFILE=$FILE

if [ "${BUNDLENAME}" == "" ]
then
    FILE=${TMPDIR}/${APPNAME}.app
else
    FILE=${TMPDIR}/${BUNDLENAME}.app
fi

cp -Rp "${ORIGFILE}" "${FILE}"

if [ "${IPANAME}" == "" ]
then
    IPANAME=$APPNAME.ipa
else
    if [ "$(echo $IPANAME | cut -b 1)" == "/" ]
    then
        OUTPUTDIR=$(dirname "${IPANAME}")/
    else
        OUTPUTDIR=$(pwd)/
    fi
    IPANAME=$(basename "${IPANAME}")
fi

if [ "${BUNDLENAME}" != "" ]
then
    EXENAME=$($PLIST -c "Print CFBundleExecutable " "${FILE}/Info.plist")
    $PLIST -c "Set CFBundleName ${BUNDLENAME}" "${FILE}/Info.plist"
    $PLIST -c "Set CFBundleExecutable ${BUNDLENAME}" "${FILE}/Info.plist"
    mv "${FILE}/${EXENAME}" "${FILE}/${BUNDLENAME}"
fi

if [ "${BUNDLEDISPLAY}" != "" ]
then
    $PLIST -c "Set CFBundleDisplayName ${BUNDLEDISPLAY}" "${FILE}/Info.plist"
fi

if [ "${BUNDLEID}" == "" ]
then
    BUNDLEID=$($PLIST -c "Print CFBundleIdentifier" "${FILE}/Info.plist")
else
    $PLIST -c "Set CFBundleIdentifier ${BUNDLEID}" "${FILE}/Info.plist"
fi

if [ "$ACCTCODE" != "" ]
then
    $PLIST -c "Set accountCode \"$ACCTCODE\"" "$FILE/Info.plist" || $PLIST -c "Add accountCode string \"$ACCTCODE\"" "$FILE/Info.plist" || cleanupAndExit -1
fi

# write plist settings passed in
echo "--- Updating PLIST Settings ---"
IFS=','
for pref in $SETTINGS;
do
    IFS='='
    read -a PARTS <<< "$pref"
    echo "(${PARTS[0]}) -> (${PARTS[1]})"
    $PLIST -c "Set ${PARTS[0]} ${PARTS[1]}" "${FILE}/Info.plist" || $PLIST -c "Add ${PARTS[0]} string \"${PARTS[1]}\"" "${FILE}/Info.plist" || cleanupAndExit -2
    IFS=','
done
unset IFS

if [ "$ACCTCODE" == "" ]
then
    $PLIST -c "Delete accountCode" "$FILE/Info.plist" || true
fi

if [ "${#BUNDLEFILES[@]}" != "0" ]
then
    pushd "$FILE" > /dev/null 2>&1

    cp -af "${BUNDLEFILES[@]}" .

    popd > /dev/null 2>&1
fi

if [ "${#CONFIGFILES[@]}" != "0" ] || [ "$TTCONFIG" != "" ]
then
    pushd "$FILE" > /dev/null 2>&1

    EXISTINGCONFIG=(*.ttconfig)

    if [ "$TTCONFIG" != "" ]
    then
        #remove all existing configurations if we're replacing it with another.
        for configFile in "${EXISTINGCONFIG[@]}"
        do
            rm -f "$configFile"
        done
    else
        TTCONFIG=$(absolutePathForFile "${CONFIGFILES[1]}")
    fi

    TTCONFIGTMP=$(mktemp -d ttconfigXXX)

    pushd "$TTCONFIGTMP" > /dev/null 2>&1

    unzip "$TTCONFIG" || cleanupAndExit $?

    if [ "${#CONFIGFILES[@]}" != "0" ]
    then
        cp -af "${CONFIGFILES[@]}" .
    fi

    if [ ! -f import.settings ]
    then
    cat > import.settings << EOF
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
<key>com.censoft.overwrite</key>
<true/>
<key>com.censoft.removeAll</key>
<false/>
<key>com.censoft.silent</key>
<false/>
</dict>
</plist>
EOF
    fi
    zip ../default.ttconfig * || cleanupAndExit $?

    popd > /dev/null 2>&1

    rm -rf "$TTCONFIGTMP"

    popd > /dev/null 2>&1
fi

ENTITLEMENTSFILE=`mktemp entitlementsXXX`

sed -e "s/@@ID@@/${BUNDLEID}/g" -e "s/@@PREFIX@@/${PREFIX}/g" > "${ENTITLEMENTSFILE}" << EOF
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
<key>application-identifier</key>
<string>@@PREFIX@@.@@ID@@</string>
<key>get-task-allow</key>
<false/>
<key>keychain-access-groups</key>
<array>
<string>@@PREFIX@@.@@ID@@</string>
</array>
</dict>
</plist>
EOF

rm -rf "$FILE/_CodeSignature" > /dev/null
rm -rf "$FILE/Frameworks/"*"/_CodeSignature" > /dev/null

# Convert Info.plist to binary
plutil -convert binary1 "$FILE/Info.plist"

# sign frameworks

if [ -d "$FILE/Frameworks" ]
then
    "${CODESIGN}" -f --deep -s $SIGNID "$FILE/Frameworks/"*
fi

#sign main app

"${CODESIGN}" -f --deep -s $SIGNID --entitlements "${ENTITLEMENTSFILE}" --generate-entitlement-der "${FILE}" || cleanupAndExit -3

pushd "$FILE/Frameworks"
for FRAMEWORK in *.framework
do
    echo Signing $FRAMEWORK
    /usr/bin/codesign --force --deep --preserve-metadata=identifier,entitlements --generate-entitlement-der --sign "$SIGNID" "$FRAMEWORK"
done
popd

rm -f "${ENTITLEMENTSFILE}"

"$PACKAGEAPP" -v "$FILE" -o "$OUTPUTDIR$IPANAME" -embed "$PROVISION" -sign "$SIGNID" || cleanupAndExit -2

cleanup

echo $OUTPUTDIR$IPANAME
exit 0
